<?php
/*
Plugin Name: 无觅相关文章插件
Plugin URI: http://wordpress.org/extend/plugins/wumii-related-posts/
Author: Wumii Team
Version: 1.0.3.10
Author URI: http://www.wumii.com
Description: Automatically finds out related articles based on article content and user behavior.
 
Copyright 2010 wumii.com (email : team[at]wumii.com)
*/

if (!class_exists('WumiiRelatedPosts')) {
    class WumiiRelatedPosts {
        const ADMIN_OPTION_NAME = 'wumii_admin_options';
        const PLUGIN_PATH = '/wp-content/plugins/wumii-related-posts';
        
        // admin option names
        const NUM_POSTS = 'num_posts';
        const SINGLE_PAGE_ONLY = 'single_page_only';
        const ENABLE_CUSTOM_POS = 'enable_custom_pos';
        
        private $adminOptions;
        private $excludePostIds = array();
        
        function WumiiRelatedPosts() {
            $this->initAdminOptions();
        }
        
        private function initAdminOptions() {
            $this->resetAdminOptions();
            
            $persistentOptions = get_option(self::ADMIN_OPTION_NAME);
            if (is_array($persistentOptions)) {
                foreach ($this->adminOptions as $key => $value) {
                    if (array_key_exists($key, $persistentOptions)) {
                        $this->adminOptions[$key] = $persistentOptions[$key];
                    }
                }
            }
        }
        
        function addCssFile() {
            wp_enqueue_style('wumii_style', self::PLUGIN_PATH . '/css/wumii.css');
        }
        
        function registerOptionsPage () {
            add_options_page('无觅相关文章插件', '无觅', 'activate_plugins', basename(__FILE__), array($this, 'printAdminPage'));
        }
        
        function addWumiiContent($content) {
            $newContent = $content;
            if ($this->canDisplayWumiiContent()) {
                $escapedUrl = htmlspecialchars(get_permalink());
                $escapedTitle = htmlspecialchars(the_title('', '', false));
                $escapedPic = htmlspecialchars($this->getPostThumbnailSrc());
                
                // The first line in 'WUMII_HOOK' must be an empty line. Because some blogs use 'Embeds'(http://codex.wordpress.org/Embeds)
                // in the post content and the embed must be on its own line.
                // For example, if the 'embed' happen to add before our wumii code,
                // then we have to make sure wumii code doesn't follow that within the same line.
                $newContent .= <<<WUMII_HOOK
                
                    <div class="wumii-hook">
                        <input type="hidden" name="wurl" value="$escapedUrl" />
                        <input type="hidden" name="wtitle" value="$escapedTitle" />
                        <input type="hidden" name="wpic" value="$escapedPic" />
                    </div>
WUMII_HOOK;
                
                global $wp_query;
                if ($wp_query->current_post + 1 == $wp_query->post_count) {
                    $blogUrl = get_bloginfo('url');
                    $enableCustomPos = $this->getAdminOption(self::ENABLE_CUSTOM_POS) ? 'true' : 'false';
                    $numPosts = $this->getAdminOption(self::NUM_POSTS);
                    
                    $newContent .= "<script><!--
                                       var wumiiSitePrefix = '$blogUrl';
                                       var wumiiEnableCustomPos = $enableCustomPos;
                                   //--></script>" .
                                   "<script src='http://widget.wumii.com/ext/relatedItemsWidget.htm?type=1&num=$numPosts'></script>" .
                                   "<a href='http://www.wumii.com/widget/relatedItems.htm' style='border:0;'><img src='http://static.wumii.com/images/pixel.png' alt='无觅相关文章插件' style='border:0;padding:0;margin:0;' /></a>";
                }
            }
            return $newContent;
        }
        
        private function getPostThumbnailSrc() {
            // function 'get_post_thumbnail_id' need blog support
            if (!function_exists('get_post_thumbnail_id')) {
                return;
            }
            
            $image_info = wp_get_attachment_image_src(get_post_thumbnail_id(),'full');
            if ($image_info) {
                return $image_info[0];
            }
        }
        
        private function canDisplayWumiiContent() {
            // We can identify the live-blogging post by checking if Live Blogging plugin is activated and the shortcode in the content.
            // The related items will show in such live blogging post only one time generally,
            // but if the theme call the filter on 'the_content' hook or run another 'The Loop' to fetch the post content using in some other cases before to display,
            // the related items will not show.
            // In brief, the related items show one time at most in each post.
            global $post;
            if (array_key_exists($post->ID, $this->excludePostIds)) {
                return false;
            }
            
            global $shortcode_tags; // Container for storing shortcode tags and their hook to call for the shortcode
            if (!empty($shortcode_tags) && is_array($shortcode_tags)
                    && array_key_exists('liveblog', $shortcode_tags) && strpos(get_the_content(), '[liveblog]') !== false) {
                $this->excludePostIds[$post->ID] = 1;
            }
            
            return get_post_type() == 'post' && // In some pages e.g. "attachment page" should not display related items
                   !is_preview() && // When create a new post and press the preview button before publish it, 
                                    // the post's permalink is not the correct form as the setting in "Permalink".
                                    // We have to prevent the related items displaying in these pages.
                   !is_feed() && 
                   !is_page() && // In some pages e.g. "about me" also should not display.
                   (is_single() || !$this->getAdminOption(self::SINGLE_PAGE_ONLY));
        }
        
        private function resetAdminOptions () {
            $this->adminOptions = array(
                self::NUM_POSTS => 4,
                self::SINGLE_PAGE_ONLY => false,
                self::ENABLE_CUSTOM_POS => false);
        }
        
        function saveAdminOptions() {
            update_option(self::ADMIN_OPTION_NAME, $this->adminOptions);
        }
        
        function getAdminOption($key) {
            return $this->adminOptions[$key];
        }
        
        // echo true will be 1, echo false will be 0
        // But echo !true will be null. So it have to strictly convert false to '0'.
        private function getBooleanInStr($boolean) {
            return $boolean ? '1' : '0';
        }
        
        private function getHtmlAdminOptionsPanel() {
            $adminOptionTemplate = new Template('adminOptionsPanel.html');
            
            $singlePageOnly = $this->getAdminOption(self::SINGLE_PAGE_ONLY);
            $enableCustomPos = $this->getAdminOption(self::ENABLE_CUSTOM_POS);
            
            $replacements = array(
                'request_uri' => $_SERVER["REQUEST_URI"],
                
                'num_posts' => $this->getAdminOption(self::NUM_POSTS),
                
                'single_page_only_checked_' . $this->getBooleanInStr($singlePageOnly) => 'checked="checked"',
                'single_page_only_checked_' . $this->getBooleanInStr(!$singlePageOnly), '',
                
                'enable_custom_pos_checked_'. $this->getBooleanInStr($enableCustomPos) => 'checked="checked"',
                'enable_custom_pos_checked_' . $this->getBooleanInStr(!$enableCustomPos), '',
            );
            
            $adminOptionTemplate->addReplacements($replacements);
            
            return $adminOptionTemplate->render();
        }
        
        
        function printAdminPage() {
            if (array_key_exists('update_wumii_settings', $_POST)) {
                foreach ($this->adminOptions as $key => $value) {
                    if (array_key_exists($key, $_POST)) {
                        $this->adminOptions[$key] = $_POST[$key];
                    }
                }
                $submitResult = '设置已更新.';
            } else if (array_key_exists('reset_wumii_settings', $_POST)) {
                $this->resetAdminOptions();
                $submitResult = '设置已重置.';
            }
            
            $this->saveAdminOptions();
            
            $htmlAdminOptionsPanel = $this->getHtmlAdminOptionsPanel();
            
            if ($submitResult) {
                echo '<div class="updated"><p><strong>' . $submitResult . '</strong></p></div>';
            }
            echo $htmlAdminOptionsPanel;
        }
        
        function finalize() {
            delete_option(self::ADMIN_OPTION_NAME);
        }
    }
    
    class Template {
        private $htmlStrTemplate;
        private $keyValuePair = array();
        
        function Template($templateName) {
            $this->htmlStrTemplate = file_get_contents($this->wumii_get_config_file_path() . WumiiRelatedPosts::PLUGIN_PATH . '/templates/' . $templateName);
        }
        
        function addReplacements($keyValuePair) {
            foreach ($keyValuePair as $key => $value) {
                $this->keyValuePair[$this->getReplacementStr($key)] = $value;
            }
        }
        
        function render() {
            return str_replace(array_keys($this->keyValuePair), array_values($this->keyValuePair), $this->htmlStrTemplate);
        }
        
        private function getReplacementStr($key) {
            return '{{' . $key . '}}';
        }
        
        private function wumii_get_config_file_path() {
            $path_index = dirname(__FILE__);
            preg_match("/^(.+)wp-content.+$/Ui", $path_index, $matches);
            // Here if $matches[0] == 'C:/workspace/Ing/wordpress/wp-content/plugins/wumii-related-posts/php/util.php', than matches[1] == 'C:/workspace/Ing/wordpress/'.
            return $matches[1];
        }
    }
    
    $wumii_related_posts = new WumiiRelatedPosts();
    
    add_action('admin_print_styles', array($wumii_related_posts, 'addCssFile'));
    add_action('admin_menu', array($wumii_related_posts, 'registerOptionsPage'));
    //  We need to place our content as near to the front as possible, so set the priority 1.
    add_action('the_content', array($wumii_related_posts, 'addWumiiContent'), 1);
    add_action('deactivate_wumii-related-posts/wumii-related-posts.php', array($wumii_related_posts, 'finalize'));
} else {
    function classConflictException() {
        echo '<div class="error"><p><strong>插件冲突。</strong>您的博客正在运行一个与“无觅相关文章插件”定义了相同类名的插件，只有在关闭冲突插件以后“无觅相关文章插件”才能正常启用。</p></div>';
    }
    add_action('admin_notices', 'classConflictException');
}
?>